#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

/* Join N strings.
   Warning: handling of NULL may change in the future. 
   At present, if _any_ of the strings are NULL, 
   NULL is returned.
   
   This function came into being because
   joining strings using a join-two-string function
   leads to very verbose error checking code in
   callers which want to join 3 or more strings 
   and every way I could think of doing it 
   compositionally would cause memory leaks under
   certain conditions. 
   
   strcount is the number of strings to join.
   The argument list is traversed twice, so that
   realloc will never need to be used. */

#ifndef FORSATABS
char *strjoin(int strcount, ...) {
	int total_len = 0; 
	int count, tmp_len;
	va_list argp;
	char *tmp, *new_string, *new_string_offset;

	va_start(argp, strcount);
	for (count = 0; count < strcount; count++) {
		tmp = va_arg(argp, char *);
		if (tmp == NULL) return NULL;
		total_len += strlen(tmp);
	}
	va_end(argp);

	total_len += 1; /* make space for terminating '\0' */
		
	new_string = malloc(total_len);
	if (new_string == NULL) return NULL;
	new_string_offset = new_string;
	
	va_start(argp, strcount);
	for (count = 0; count < strcount; count++) {
		tmp = va_arg(argp, char *);
		tmp_len = strlen(tmp);
		strncpy(new_string_offset, tmp, tmp_len);
		new_string_offset += tmp_len;
	}
	va_end(argp);
	strncpy(new_string_offset, "", 1);

	return new_string;
}
#endif /* FORSATABS */


/* This should be functionally equivalent to strcmp.
   The reason that it is provided is so that satabs can have
   a function body to reason about, because otherwise the 
   semantics of strcmp are lost and a non-deterministic 
   return value is considered instead, wrecking havoc
   on the verification. 
   The following definition of (c)strcmp appears to be usable
   in code which needs to compare strings and be verified.*/
int cstrcmp(const char *s0, const char *s1) {
        /* the following semantics for the ordering of a NULL
           and non-NULL string are highly questionable, but oh well.*/
        int index;
        if ((s0 == NULL) || (s1 == NULL)) return !(s0 == s1);

        index = 0;
        while((s0[index] != '\0') && (s1[index] != '\0')) {
                if (s0[index] != s1[index]) {
                        return s0[index] > s1[index] ? 1 : -1;
                }
                index++;
        }
        if (s0[index] != s1[index]) return s0[index] ? 1 : -1;
        return 0;
}

/* inet_addr is obsolete, but also does what's needed .. 
   This version is wrong: it doesn't even try to 
   convert to network byte order.
*/

/*
int cinet_addr(char *ipv4_addr) {
	int result, i, s_i;
	char cur_octet[4];

	for (i = 0, s_i = 0; i < 4; i++) {
		cur_octet =  
	}

}*/


