/* This is the central update server, which clients
   register with, and which pushes updates out to them
   (in the form of agents) when updates become available.
 */

#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h> /* for ssize_t */

#include "agentxfer.h"
#include "listener.h"
#include "wrappers.h"

#define MAX_REMOTE_INFO_SIZE 200
#define MAX_CLIENTS 2
#define UPDATE_NAME "client_update.tar.gz"

char *clients[MAX_CLIENTS];
int client_count = 0;

void mainloop(int sockfd);
void send_update(void);
void send_update_to(char *location);

int main(int argc, char **argv) {
	int sockfd = init_listener(argc, argv);
	mainloop(sockfd);
}

void mainloop(int sockfd) {
	ssize_t bytes_read;
	char remote_info[MAX_REMOTE_INFO_SIZE];	

	while(1) {
		sleep(1);
		send_update(); /* try to send an update to all clients every second... */
		bytes_read = wget_data(sockfd, remote_info, MAX_REMOTE_INFO_SIZE);
		if (bytes_read == -1) {
			warn(describe_err(errno));
			continue;
		}
		/* assume that the above was a null-terminated string representing a location */
		if (client_count >= MAX_CLIENTS) continue;
		clients[client_count] = strdup(remote_info);
		client_count++;
		warn("Addded client %s\n", remote_info);
	}
}

void send_update(void) {
	int i;
	for (i = 0; i < client_count; i++) {
		send_update_to(clients[i]);
	}
}

/* the updating code is an agent/capplet */
void send_update_to(char *location) {
	send_agent_to(location, UPDATE_NAME);
}


