#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>

#ifndef FORSATABS
#include <fcntl.h> /* for O_RDONLY */
#endif

#include "wrappers.h"
#include "filexfer.h"

#define BUFSIZE 1024


/* this is quite inefficient, and doesn't deal well with error
 * conditions.  Should be renamed. */
int send_file(int sockfd, char *filename) {
    int fd;
    ssize_t sent_amount, read_amount;
    char data[BUFSIZE];

    fd = open(filename, O_RDONLY);
    if (fd == -1) return -1;

    while((read_amount = read(fd, data, BUFSIZE)) != 0) {
        if (read_amount == -1) {
            if (errno == EINTR) continue;
			warn("Reading from %s failed. Cancelling send.\n", filename);
            wclose(fd);
            return -1;
        }
        sent_amount = send(sockfd, data, read_amount, 0);
		if (sent_amount == -1) { /* handle EINTR */
			warn("Sending failed (%s). Cancelling send.\n", describe_err(errno));
			wclose(fd);
			return -1;
		}
        if (sent_amount != read_amount) {
            warn("handle this case.. sent_amount != read_amount\n");
        }
    }
    wclose(fd);
    return 0;
}


/* another ineffient file handling function which deals
 * poorly with error conditions. Should be renamed. */
int recv_file(int sockfd, char *req_name, char *save_name) {
    ssize_t sent;
    int save_fd, recv_retval;

    sent = rsend(sockfd, req_name, strlen(req_name) + 1, 0);
    if (sent == -1) {
        printf("Unable to transmit request for file %s\n", req_name);
        show_error_and_exit();
    }

    save_fd = open(save_name, O_WRONLY|O_CREAT);
    if (save_fd == -1) {
        printf("Unable to open file %s for writing\n", save_name);
        show_error_and_exit();
    }

	recv_retval = recv_file_fd(sockfd, save_fd);
    wclose(save_fd);
    return recv_retval;
}

int recv_file_fd(int sockfd, int filefd) {
	ssize_t bytes_recv, bytes_written;
	char file_buffer[BUFSIZE];
	while((bytes_recv = recv(sockfd, file_buffer, BUFSIZE, 0)) != 0) {
		if (bytes_recv == -1) {
			if (errno == EINTR) continue; // try again
			warn("Giving up download.\n");
			return -1;
		}
		bytes_written = write(filefd, file_buffer, bytes_recv);
		if (bytes_written < bytes_recv) { /* handle EINTR too */
			warn("handle this case... sent-amount != read_amount\n");
		}
	}
	return 0;	
}
