module ParseInitLoc where

-- this should share a definition of
-- location and host with the security policy code.

import Text.ParserCombinators.Parsec
import Text.ParserCombinators.Parsec.Perm
--import Text.ParserCombinators.Parsec.TokenParser as P
import Data.Tree

rootLocation = Location "env" "127.0.0.1" 0
invalidLocation = Location "" "" 0

getInitLocInfo :: String -> Either ParseError InitLocInfo
getInitLocInfo input = 
  parse findLocInfo "" input


getLocationNet :: String -> Either ParseError  (Tree Location)
getLocationNet input =
  parse findLocNet "" input

findLocInfo = do
  spaces
  locNet <- findLocNet
  spaces
  curLoc <- findCurLoc
  spaces
  cbh <- findCodeBaseHost
  spaces
  co <- findCodeOrigin
  return $ InitLocInfo locNet curLoc cbh co

 
--findCurLoc = manyTill anyChar (try (string "\n"))
findLocNet = do
  string "location net:"
  spaces
  topLevelSubtrees <- findLocationNetForest
  return $ Node rootLocation topLevelSubtrees

-- fixme: use parens here
findLocationNetForest :: Parser (Forest Location)
findLocationNetForest = do
  char '('
  spaces
  forest <- many1 findLocationTree
  spaces
  char ')'
  spaces
  return forest

findLocationTree :: Parser (Tree Location)
findLocationTree = do
  root <- findLocationComponent
  spaces
  (host, port) <- findHostAndPort
  spaces
  subForest <- findLocationNetForest <|> findEmptyForest
  return $ Node (Location root host port) subForest

findEmptyForest :: Parser (Forest Location)
findEmptyForest = return []

findLocationComponent = many1 (alphaNum <|> char '.')

findHostAndPort :: Parser (String, Int)
findHostAndPort = do
  char '['
  spaces
  host <- many1 (digit <|> char '.')
  spaces
  port <- many1 digit
  spaces
  char ']'
  return (host, read port)

{-
-- fixme: use the definition
-- located in the security policy grammar
findLocationComponent :: Parser Tree
findLocationComponent = do
	locationComponent <- many1 (alphaNum <|> '.')
	return $ Node locationComponent [] -- [] as it's a leaf
	-- nm, it may or may not be.
-}


findCurLoc = findField "current execution location:"

findCodeOrigin = findField "code origin:"

findCodeBaseHost = findField "codebase host:"


findField field = do
  string field
  spaces
  many1 (alphaNum <|> char ':' <|> char '.') -- FIX this definition: allowedChar <|> char ':') -- FIX this definition: allowedCharss

--type Location = String
data Location = Location {locName::String, locIP::String, locPort::Int} deriving Show

-- FIXME: two of the following String types should be Location
data InitLocInfo = InitLocInfo {iliLocationNet::Tree Location, iliCurrentLocation::String,
                                iliCodeBaseHost::String, iliCodeOrigin::String} deriving Show


