/* push-recv is paired with push-sender. 
 * It waits, listening on a port, for incoming file transfers, 
 * executes the code from them, and continues listening.
 */

#include <stdlib.h>
#include <sys/socket.h> /* for socklen_t */
#include <arpa/inet.h> /* for struckt sockaddr_in */
#include <errno.h>
#include <string.h>

#include "wrappers.h"
#include "listener.h"
#include "filexfer.h"
#include "run.h"
#include "clocation.h"
#include "push_recv_core.h"

void mainloop(int sockfd) {
	#define TEMPLATE_LEN 11
	int connfd, filefd;
	socklen_t struct_size;
	struct sockaddr_in ri;
	char template[TEMPLATE_LEN]; /* needed for mkstemp */
	 /* 10 chars + '\0'; 6 X's as per the man page.
	  * Do NOT make this longer without changing TEMPLATE_LEN, 
	  * else template may not end up NUL terminated (see man strncpy) */
	char *filename_template = "cappXXXXXX";
	while(1) {
		strncpy(template, filename_template, TEMPLATE_LEN);
		struct_size = sizeof(struct sockaddr_in);
		connfd = accept(sockfd, (struct sockaddr *) &ri, &struct_size);
		if (connfd == -1) {
			warn("Accept failed.  Continuing.\n");
			continue;
		}

		filefd = mkstemp(template);
		if (filefd == -1) {
			warn("Couldn't create temp file %s (%s), skipping download from %s\n",
						template, describe_err(errno), inet_ntoa(ri.sin_addr));
			wclose(connfd);
			continue;
		}
		if (recv_file_fd(connfd, filefd) == -1) {
			warn("Download from %s failed (%s).  Not trying to execute.\n", 
							inet_ntoa(ri.sin_addr), describe_err(errno));
			wclose(connfd);
			wclose(filefd);
			continue;
		}
		wclose(filefd); /* mkstemp opens it with an exclusive lock */
		wclose(connfd); /* no further communication */
		set_inet_origin("127.0.0.1", "5555", "", template); /* FIXME: don't hardcode port....! */
		run_agent(template);
		warn("Location: %s\n", get_capplet_origin(template));
	}
}


