#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdarg.h>

#ifndef FORSATABS
#include <fcntl.h> /* for O_RDONLY */
#endif

#include "wrappers.h"
#include "filexfer.h"
#include "listener.h"

void mainloop(int sockfd);
int verify_filename(char *filename);
extern int debug;

int main(int argc, char **argv) {
	int socketfd = init_listener(argc, argv);
	mainloop(socketfd);
	return 0;
}

void mainloop(int sockfd) {
	int connfd, bytes_rec;
	socklen_t struct_size;
	struct sockaddr_in ri;
	char filename[MAX_FILENAME_LENGTH];
	
	while (1) {
		/* reset struct_size every time: it's an upper bound, 
		 * and accept may change the value */
		struct_size = sizeof(struct sockaddr_in);
		connfd = accept(sockfd, (struct sockaddr *) &ri, &struct_size);
		if (connfd == -1) {
			warn("Accept failed.  Continuing.\n");
			continue;
		} 
		
		bytes_rec = recv(connfd, &filename, MAX_FILENAME_LENGTH - 1, 0);
		if (bytes_rec <= 0) {
			warn("Warning: no filename. %s\n", bytes_rec == 0? "Connection closed." : describe_err(errno));
			wclose(connfd);
			continue; 
		}

		if (bytes_rec >= MAX_FILENAME_LENGTH) {
			/* impossible case: here to make satabs happy. */
			bytes_rec = MAX_FILENAME_LENGTH - 1;
		}
		filename[bytes_rec] = '\0'; /* guarantee NUL termination */
		if (!verify_filename(filename)) {
			warn("Rejected request for %s\n", filename);
			continue;
		}
		if (send_file(connfd, filename) == -1) {
			warn("Sending %s failed.\n", filename);		
		} else {
			warn("Transfer of %s finished successfully.\n", filename);
		}
		wclose(connfd);
	}
}

int verify_filename(char *filename) {
	return 1;
}


